#include <WScom.h>
#include <WSCfunctionList.h>
#include <WSCbase.h>
#include <WSCmessageDialog.h>
#include "dssbp.h"

extern WSCmessageDialog* newmess_000;

#define SAMPLE_RATE 48000
#define PA_SAMPLE_TYPE paFloat32
#define FRAMES_PER_BUFFER 64

typedef float SAMPLE;

PortAudioStream *pa_stream;
Detector *detector;
Generator *generator;
int filter_changed = 0;
float ifreq = 12000.0f;
int ifreq_changed = 0;
float low_cutoff = 200.0f;
float high_cutoff = 3000.0f;
float attn = 60.0f;
float tr_bw = 200.0f;
int running = 0;
int tone_p = 0;
int generating = 0;
int mode = 1;

SinGen *tone_gen;

//----------------------------------------------------------
//Function for the event procedure
//----------------------------------------------------------
/* This will be called asynchronously by the PortAudio engine. */

static int paCallback(const void *inputBuffer, void *outputBuffer,
		      unsigned long framesPerBuffer,
		      const PaStreamCallbackTimeInfo* timeInfo,
		      PaStreamCallbackFlags statusFlags,
		      void *userData)
{
  SAMPLE *out = (SAMPLE*)outputBuffer;
  const SAMPLE *in = (const SAMPLE*)inputBuffer;
  static SAMPLE input[FRAMES_PER_BUFFER], detout[FRAMES_PER_BUFFER], genout_l[FRAMES_PER_BUFFER], genout_r[FRAMES_PER_BUFFER];
  static float itone[FRAMES_PER_BUFFER], qtone[FRAMES_PER_BUFFER];
  SAMPLE *p = input;
  SAMPLE *p_l = NULL;
  SAMPLE *p_r = NULL;
  unsigned int i;
  float leftInput, rightInput;
  (void) timeInfo; /* Prevent unused variable warnings. */
  (void) statusFlags;
  (void) userData;

  if (inputBuffer == NULL) return 0;

  if (running == 1) {
    if (tone_p) {
      tone_gen->get_sample_n(itone, qtone, framesPerBuffer);
      p = itone;
    } else {
      /* Get interleaved samples from input buffer. */
      // detector->detect(in, out, framesPerBuffer);
      for (i = 0; i < framesPerBuffer; i++) {
	in += 2;		// use only 3rd input channel
	*p++ = *in++;
	rightInput = *in++;		// throw away
      }
      p = input;
    }

    generator->generate(p, genout_l, genout_r, framesPerBuffer);
    
    p_l = genout_l;
    p_r = genout_r;
    for (i = 0; i < framesPerBuffer; i++) {
      //fprintf(stderr, "%f\t%f\n", *p_l, *p_r);
      out += 2;
      *out++ = *p_l++;
      *out++ = *p_r++;
    }
  } else {
  }

  if (filter_changed) {
    generator->set_iqfilter(12000.0f, low_cutoff, high_cutoff, tr_bw, attn);
    detector->set_iqfilter(12000.0f, low_cutoff, high_cutoff, tr_bw, attn);
    filter_changed = 0;
  }
  if (ifreq_changed) {
    generator->set_ifreq(ifreq);
    //detector->set_ifreq(ifreq);
    ifreq_changed = 0;
  }
  return paContinue;
}

void init_proc(WSCbase* object){
  PaStreamParameters inputParameters, outputParameters;
  PaError err;

  err = Pa_Initialize();
  if (err != paNoError) {
    popup_pa_err_dialog(err);
  }

  inputParameters.device = Pa_GetDefaultInputDevice();
  //inputParameters.device = (PaDeviceIndex)12;
  //inputParameters.device = (PaDeviceIndex)18;
  if (inputParameters.device == paNoDevice) {
    char *errmsg = "No such input device found.";
  
    Pa_Terminate();
    newmess_000->setProperty(WSNlabelString, errmsg);
    newmess_000->popup();
  }
  inputParameters.channelCount = 4;       /* 4 inputs */
  inputParameters.sampleFormat = PA_SAMPLE_TYPE;
  inputParameters.suggestedLatency = Pa_GetDeviceInfo(inputParameters.device)->defaultLowInputLatency;
  inputParameters.hostApiSpecificStreamInfo = NULL;

  outputParameters.device = Pa_GetDefaultOutputDevice();
  //outputParameters.device = (PaDeviceIndex)17;
  //outputParameters.device = (PaDeviceIndex)18;
  if (inputParameters.device == paNoDevice) {
    char *errmsg = "No such input device found.";
    
    Pa_Terminate();
    newmess_000->setProperty(WSNlabelString, errmsg);
    newmess_000->popup();
  }
  outputParameters.channelCount = 4;       /* 4 outputs */
  outputParameters.sampleFormat = PA_SAMPLE_TYPE;
  outputParameters.suggestedLatency = Pa_GetDeviceInfo(outputParameters.device)->defaultLowOutputLatency;
  outputParameters.hostApiSpecificStreamInfo = NULL;

  err = Pa_OpenStream(
		      &pa_stream,
		      &inputParameters,
		      &outputParameters,
		      SAMPLE_RATE,
		      FRAMES_PER_BUFFER,
		      0, /* paClipOff, */
		      paCallback,
		      NULL);

  if (err != paNoError) {
    popup_pa_err_dialog(err);
  }

  // create a Detector object
  detector = new Detector();
  detector->detecting = 1;
  detector->mode = 1;

  // create a Generator object
  generator = new Generator();
  generator->generating = 1;
  generator->mode = 1;
  //
  tone_gen = new SinGen(1000, 48000, 0.1);
  //tone_gen = new SinGen(1000, 48000);
}

static WSCfunctionRegister  op("init_proc",(void*)init_proc);
